//  -*- Mode: c++ -*-
// ============================================================================
// 
// Copyright (c) 1997 The CGAL Consortium
//
// This software and related documentation is part of an INTERNAL release
// of the Computational Geometry Algorithms Library (CGAL). It is not
// intended for general use.
//
// ----------------------------------------------------------------------------
//
// release       : $CGAL_Revision: CGAL-1.0 $
// release_date  : $CGAL_Date: 1998/09/12 $
//
// file          : include/CGAL/bops_Triangle_2.C
// source        : include/CGAL/bops_Triangle_2.C
// revision      : $Revision: 1.1.2 $
// revision_date : $Date: Wed Dec  9 13:28:47 MET 1998  $
// author(s)     :             Wolfgang Freiseisen <Wolfgang.Freiseisen@risc.uni-linz.ac.at>
//
// coordinator   : RISC Linz
//  (Wolfgang Freiseisen <wfreisei@risc.uni-linz.ac.at>)
//
// 
// ============================================================================

#ifndef CGAL_BOPS_TRIANGLE_2_C
#define CGAL_BOPS_TRIANGLE_2_C

//#define CGAL__BOPS_USE_STANDARD_CODE
   // if CGAL__BOPS_USE_STANDARD_CODE is defined, then the
   // code for simple polygons will be used for computing 
   // BOPS on iso-rectangles and triangles.

#include <CGAL/bops_simple_polygons_2.h>
#include <CGAL/bops_Container_Polygon_2.h>

#include<CGAL/bops_Triangle_2.h>
#include <CGAL/bops_assertions.h>

/*
#include <CGAL/Triangle_2_Triangle_2_intersection.h>
bool CGAL_do_intersect( const CGAL_Triangle_2<R>& A,
                        const CGAL_Triangle_2<R>& B);
CGAL_Object CGAL_intersection( const CGAL_Triangle_2<R>& A,
                        const CGAL_Triangle_2<R>& B)
see 'CGAL/Triangle_2_Triangle_2_intersection.h'
*/
 

template < class R, class OutputIterator >
OutputIterator CGAL_union( const CGAL_Triangle_2<R>& A,
                           const CGAL_Triangle_2<R>& B,
		           OutputIterator result)
{
  CGAL_bops_precondition_msg( !A.is_degenerate(),
                              "Triangle_2<R> A is degenerated");
  CGAL_bops_precondition_msg( !B.is_degenerate(),
                              "Triangle_2<R> B is degenerated");

  Bops_default_I<R> default_traits;
  typedef typename Bops_default_I<R>::Input_polygon_container input;
  input cA, cB;
  back_insert_iterator<input> inA(cA), inB(cB);
  for(int i= 0; i<3; i++) { *inA++ = A[i]; *inB++ = B[i]; }

  return CGAL_union( cA.begin(), cA.end(), cB.begin(), cB.end(),
	 default_traits, result );
}
 
 
template < class R, class OutputIterator >
OutputIterator CGAL_difference( const CGAL_Triangle_2<R>& A,
		                const CGAL_Triangle_2<R>& B,
		                OutputIterator result)
{
  CGAL_bops_precondition_msg( !A.is_degenerate(),
                              "Triangle_2<R> A is degenerated");
  CGAL_bops_precondition_msg( !B.is_degenerate(),
                              "Triangle_2<R> B is degenerated");

  Bops_default_I<R> default_traits;
  typedef typename Bops_default_I<R>::Input_polygon_container input;
  input cA, cB;
  back_insert_iterator<input> inA(cA), inB(cB);
  for(int i= 0; i<3; i++) { *inA++= A[i]; *inB++= B[i]; }

  return CGAL_difference( cA.begin(), cA.end(), cB.begin(), cB.end(),
	 default_traits, result );
} 
 

#endif // CGAL_BOPS_TRIANGLE_2_C
