//  -*- Mode: c++ -*-
// ============================================================================
// 
// Copyright (c) 1997 The CGAL Consortium
//
// This software and related documentation is part of an INTERNAL release
// of the Computational Geometry Algorithms Library (CGAL). It is not
// intended for general use.
//
// ----------------------------------------------------------------------------
//
// release       : $CGAL_Revision: CGAL-1.0 $
// release_date  : $CGAL_Date: 1998/09/12 $
//
// file          : include/CGAL/bops_dcel_out.h
// source        : include/CGAL/bops_dcel_out.h
// revision      : $Revision: 1.1.2 $
// revision_date : $Date: Wed Dec  9 13:28:52 MET 1998  $
// author(s)     :             Wolfgang Freiseisen <Wolfgang.Freiseisen@risc.uni-linz.ac.at>
//
// coordinator   : RISC Linz
//  (Wolfgang Freiseisen <wfreisei@risc.uni-linz.ac.at>)
//
// 
// ============================================================================

#ifndef DCEL_OUT_H
#define DCEL_OUT_H

#include <iostream.h>


#ifdef CGAL__DCEL_DEFS_H
ostream& operator<<(ostream& o, const CGAL__Dcel_Color& c) {
  o << int(c);
  return o;
}
#endif

#ifdef CGAL__DCEL_VERTEX_H
template <class I>
ostream& operator<<(ostream& o, const CGAL__Dcel_vertex_type<I>* e) {
  if( e != NULL )
    o << 'v' << (*e).index();
  else
    o << "nil";
  return o;
}
#endif

#ifdef CGAL__DCEL_FACE_H
template <class I>
ostream& operator<<(ostream& o, const CGAL__Dcel_face_type<I>* e) {
  if( e != NULL )
    o << 'f' << (*e).index();
  else
    o << "nil";
  return o;
}
#endif

#ifdef CGAL__DCEL_EDGE_H
template <class I>
ostream& operator<<(ostream& o, const CGAL__Dcel_edge_type<I>* e) {
  if( e != NULL )
    o << '[' << (*e).index() << ']';
  else
    o << "nil";
  return o;
}
#endif


#ifdef CGAL__DCEL_VERTEX_H
template <class I>
ostream& operator<<(ostream& o, const CGAL__Dcel_vertex_type<I>& vertex) {
  o << "V("    << vertex.index()
    << ", d"   << vertex.degree()
    << ", c"   << vertex.color()
    << ", HV=" << vertex.header()
    << ", pt=" << vertex.point() << ')';
  return o;
}
#endif

#ifdef CGAL__DCEL_FACE_H
template <class I>
ostream& operator<<(ostream& o, const CGAL__Dcel_face_type<I>& face) {
  o << "F("    << face.index()
    << ", c"   << face.color()
    << ", HF=" << face.header() << ')';
  return o;
}
#endif

#ifdef CGAL__DCEL_EDGE_H
template <class I>
ostream& operator<<(ostream& o, const CGAL__Dcel_edge_type<I>& edge) {
#ifdef CGAL__DCEL_DEBUG_ON_ALL
  o << "{ c=" << edge.color() << ", ";
  if( edge.V1() == NULL )
    o << "nil";
  else
    o << (CGAL__Dcel_vertex_type<I>)*(edge.V1());

  o << ", ";
  if( edge.V2() == NULL )
    o << "nil";
  else
    o << (CGAL__Dcel_vertex_type<I>)*(edge.V2());

  o << ", ";
  if( edge.F1() == NULL )
    o << "nil";
  else
    o << (CGAL__Dcel_face_type<I>)*(edge.F1());
  o << ", ";
  if( edge.F2() == NULL )
    o << "nil";
  else
    o << (CGAL__Dcel_face_type<I>)*(edge.F2());

  o << ", " << edge.P1() << ", " << edge.P2();

  o << '}';
#else
  o << "{ " << edge.index() << ": c" << edge.color() << ", "
    << edge.V1() << ", " << edge.V2() << ", "
    << edge.F1() << ", " << edge.F2() << ", "
    << edge.P1() << ", " << edge.P2() << '}';
#endif
  return o;
}
#endif


template<class T1, class T2>
ostream& operator<<(ostream& o, pair<T1,T2> p){
  o << p.first << ' ' << p.second;
  return o;
}


template<class Iterator>
ostream& print( ostream& o, char* text, Iterator begin, Iterator end) {
  Iterator it;
  int i= 0;
  for( it= begin; it != end; it++ )
    o << text << '[' << i++ << "]= " << *it << endl;
  return o;
}

#endif // DCEL_OUT_H
