//  -*- Mode: c++ -*-
// ============================================================================
// 
// Copyright (c) 1997 The CGAL Consortium
//
// This software and related documentation is part of an INTERNAL release
// of the Computational Geometry Algorithms Library (CGAL). It is not
// intended for general use.
//
// ----------------------------------------------------------------------------
//
// release       : $CGAL_Revision: CGAL-1.0 $
// release_date  : $CGAL_Date: 1998/09/12 $
//
// file          : include/CGAL/bops_dcel_vertex.h
// source        : include/CGAL/bops_dcel_vertex.h
// revision      : $Revision: 1.1.2 $
// revision_date : $Date: Wed Dec  9 13:28:53 MET 1998  $
// author(s)     :             Wolfgang Freiseisen <Wolfgang.Freiseisen@risc.uni-linz.ac.at>
//
// coordinator   : RISC Linz
//  (Wolfgang Freiseisen <wfreisei@risc.uni-linz.ac.at>)
//
// 
// ============================================================================

#ifndef CGAL__DCEL_VERTEX_H
#define CGAL__DCEL_VERTEX_H

#include <list.h>
#include <vector.h>

#include <CGAL/bops_dcel_element.h>


/*
  VERTEX in the DCEL:
  -------------------
  vertex_type:      point, header, color;
  vertex:           typedef const CGAL__Dcel_vertex_type* CGAL__Dcel_vertex;
  container:        vector<CGAL__Dcel_vertex_type>
  vertex_iterator:  typedef vector<CGAL__Dcel_vertex_type>::const_iterator
                    vertex_iterator;
  conversion:       vertex and vertex_iterator are type-identical

  *---------------------------------------------------------------------*
  *A CGAL_Point< R > is stored as a CGAL_Object to avoid templatizing of*
  *class CGAL__Dcel_vertex_type!                                        *
  *---------------------------------------------------------------------*

  CGAL-point:       CGAL_Point< R > pt;
                    if( CGAL_assign( pt, vertex->point()) )
                      this is a CGAL-point;
                    else
                      this is not a CGAL-point;

                    ...

  CGAL-object:      CGAL_Object obj= CGAL_make_object(pt);
  
*/

template <class I>
class CGAL__Dcel_vertex_type : public CGAL__Dcel_element_type {
public:
  typedef typename I::Point                                 Point;

#ifdef CGAL_CFG_INCOMPLETE_TYPE_BUG_4
  typedef CGAL__Dcel_point_smaller_x<Point>        Point_smaller;
  typedef set<Point, Point_smaller>                Points_container;
  typedef typename Points_container::const_iterator const_points_iterator;
  typedef typename Points_container::iterator       points_iterator;
  //typedef Point*                   const_points_iterator;
  typedef CGAL__Dcel_edge_type<I>* edge;
#else
  //typedef typename I::points_iterator       points_iterator;
  typedef typename I::const_points_iterator const_points_iterator;
  typedef typename I::const_edges_iterator  edge;
#endif

  CGAL__Dcel_vertex_type() : CGAL__Dcel_element_type(), _degree(0) {}

  CGAL__Dcel_vertex_type(
     //const Point& pt,
     const_points_iterator pt,
     int ind,
     CGAL__Dcel_Color col = CGAL__NO_COLOR
  )
     : CGAL__Dcel_element_type(ind, col),
     _point(pt), _header(NULL), _degree(0) {}

  CGAL__Dcel_vertex_type(
     //const Point& pt,
     const_points_iterator pt,
     CGAL__Dcel_Color col = CGAL__NO_COLOR)
        : CGAL__Dcel_element_type(col),
          _point(pt), _header(NULL), _degree(0) {}

  Point point() const { return *_point; }
  //Point& point() { return (Point)*(points_iterator)_point; }
  //const Point& point() const { return _point; }
  //Point& point() { return _point; }

  int  degree() const { return _degree; }
  edge header() const { return _header; }

protected:
  edge&  header() { return _header; }
  void   header(edge h)       { _header= h; }

  friend class CGAL__Dcel_base<I>;

private:
  //Point _point;
  const_points_iterator _point;
  edge _header;
  int  _degree; /* the degree of this node */
};

#endif /* CGAL__DCEL_VERTEX_H */

