//  -*- Mode: c++ -*-
// ============================================================================
// 
// Copyright (c) 1997 The CGAL Consortium
//
// This software and related documentation is part of an INTERNAL release
// of the Computational Geometry Algorithms Library (CGAL). It is not
// intended for general use.
//
// ----------------------------------------------------------------------------
//
// release       : $CGAL_Revision: CGAL-1.0 $
// release_date  : $CGAL_Date: 1998/09/12 $
//
// file          : include/CGAL/nsquare_intersecting.C
// source        : include/CGAL/nsquare_intersecting.C
// revision      : $Revision: 1.1.2 $
// revision_date : $Date: Wed Dec  9 13:28:55 MET 1998  $
// author(s)     :             Carl Van Geem <Carl.Van.Geem@risc.uni-linz.ac.at>
//
// coordinator   : RISC Linz
//  (Wolfgang Freiseisen <wfreisei@risc.uni-linz.ac.at>)
//
// 
// ============================================================================

template < class R > 
CGAL_Intersectionresult<R>::CGAL_Intersectionresult() 
      {   
        _is_vertex_of_poly1 = false;
        _is_vertex_of_poly2 = false;
        _is_edge_of_poly1 = false;
        _is_edge_of_poly2 = false;
        _is_vertex_vertex_intersection = false;
        _is_vertex_edge_intersection = false;
        _is_edge_edge_intersection = false;
      }

template < class R > 
CGAL_Intersectionresult<R>::CGAL_Intersectionresult (const 
                                        CGAL_Intersectionresult<R> &ires )
     {  
       _intersection_object = ires._intersection_object;
       _segments_poly1 = ires._segments_poly1;
       _segments_poly2 = ires._segments_poly2;
       _is_vertex_of_poly1 = ires._is_vertex_of_poly1;
       _is_vertex_of_poly2 = ires._is_vertex_of_poly2;
       _is_edge_of_poly1 = ires._is_edge_of_poly1;
       _is_edge_of_poly2 = ires._is_edge_of_poly2;
       _is_vertex_vertex_intersection = 
                 ires._is_vertex_vertex_intersection;
       _is_vertex_edge_intersection = ires._is_vertex_edge_intersection;
       _is_edge_edge_intersection = ires._is_edge_edge_intersection;
     }

template < class R > 
CGAL_Intersectionresult<R>::CGAL_Intersectionresult (
                           const CGAL_Point_2<R> &iobj,
                           const CGAL_Segment_2<R> &iseg1,
                           const CGAL_Segment_2<R> &iseg2,
                           const bool &i_is_vertex_of_poly1,
                           const bool &i_is_vertex_of_poly2,
                           const bool &i_is_edge_of_poly1,
                           const bool &i_is_edge_of_poly2,
                           const bool &i_is_vertex_vertex_intersection,
                           const bool &i_is_vertex_edge_intersection,
                           const bool &i_is_edge_edge_intersection) 
     {
       _intersection_object = CGAL_make_object(iobj);
       _segments_poly1.push_back(iseg1);
       _segments_poly2.push_back(iseg2);
       _is_vertex_of_poly1 = i_is_vertex_of_poly1;
       _is_vertex_of_poly2 = i_is_vertex_of_poly2;
       _is_edge_of_poly1 = i_is_edge_of_poly1;
       _is_edge_of_poly2 = i_is_edge_of_poly2;
       _is_vertex_vertex_intersection = i_is_vertex_vertex_intersection;
       _is_vertex_edge_intersection = i_is_vertex_edge_intersection;
       _is_edge_edge_intersection = i_is_edge_edge_intersection;
     }

template < class R > 
CGAL_Intersectionresult<R>::CGAL_Intersectionresult (
                           const CGAL_Segment_2<R> &iobj,
                           const CGAL_Segment_2<R> &iseg1,
                           const CGAL_Segment_2<R> &iseg2,
                           const bool &i_is_vertex_of_poly1,
                           const bool &i_is_vertex_of_poly2,
                           const bool &i_is_edge_of_poly1,
                           const bool &i_is_edge_of_poly2,
                           const bool &i_is_vertex_vertex_intersection,
                           const bool &i_is_vertex_edge_intersection,
                           const bool &i_is_edge_edge_intersection) 
     {
       _intersection_object = CGAL_make_object(iobj);
       _segments_poly1.push_back(iseg1);
       _segments_poly2.push_back(iseg2);
       _is_vertex_of_poly1 = i_is_vertex_of_poly1;
       _is_vertex_of_poly2 = i_is_vertex_of_poly2;
       _is_edge_of_poly1 = i_is_edge_of_poly1;
       _is_edge_of_poly2 = i_is_edge_of_poly2;
       _is_vertex_vertex_intersection = i_is_vertex_vertex_intersection;
       _is_vertex_edge_intersection = i_is_vertex_edge_intersection;
       _is_edge_edge_intersection = i_is_edge_edge_intersection;
     }


template < class R > 
CGAL_Intersectionresult<R>& CGAL_Intersectionresult<R>::operator=(
                                 const CGAL_Intersectionresult<R>  &ires)  
     {
       _intersection_object = ires._intersection_object;
       _segments_poly1 = ires._segments_poly1;
       _segments_poly2 = ires._segments_poly2;
       _is_vertex_of_poly1 = ires._is_vertex_of_poly1;
       _is_vertex_of_poly2 = ires._is_vertex_of_poly2;
       _is_edge_of_poly1 = ires. _is_edge_of_poly1;
       _is_edge_of_poly2 = ires._is_edge_of_poly2;
       _is_vertex_vertex_intersection = ires._is_vertex_vertex_intersection;
       _is_vertex_edge_intersection = ires._is_vertex_edge_intersection;
       _is_edge_edge_intersection = ires._is_edge_edge_intersection;
       return *this;
     }

template < class R > 
CGAL_Intersectionresult<R>::~CGAL_Intersectionresult() 
     { 
     }



template < class R > 
CGAL_List_of_intersections<R>::CGAL_List_of_intersections() 
{
}
template < class R > 
CGAL_List_of_intersections<R>::CGAL_List_of_intersections(
                      const CGAL_List_of_intersections<R> &ilist) 
{
       _list_of_intersections = ilist._list_of_intersections;
}
template < class R > 
CGAL_List_of_intersections<R>& CGAL_List_of_intersections<R>::operator=(
                              const CGAL_List_of_intersections<R> &ilist) 
{
       _list_of_intersections = ilist._list_of_intersections;
       return *this;
}
template < class R > 
CGAL_List_of_intersections<R>::~CGAL_List_of_intersections() 
{
}


